"""

"""
import math
import numpy as np

def mod(x, y):
    """
    Mappable modulo function
    :param x: First number
    :param y: Second number
    :return: x % y
    """
    return [a % b for a, b in zip(x, y)]


heavenly_body_radius = {  # [km]
    "earth": 6371,
    "luna": 1737,
    "mars": 3390,
    "venus": 6052,
    "mercury": 2440,
    "sol": 695700,
    "jupiter": 69911,
    "saturn": 58232,
    "uranus": 25362,
    "neptune": 24622,
    "pluto": 1188,
}

heavenly_body_mass = {  # [kg]
    "earth": 5.972*10**24,
    "luna": 73.46*10**21,
    "mars": 641.71*10**21,
    "venus": 4867.5*10**21,
    "mercury": 330.11*10**21,
    "sol": 1.9885*10**30,
    "jupiter": 1.8982*10**27,
    "saturn": 5.6834*10**26,
    "uranus": 8.6810*10**25 ,
    "neptune": 1.02413*10**26,
    "pluto": 13.03*10**21,
}

heavenly_body_period = {  # [days]
    "earth": 1,
    "luna": 27.321661,
    "mars": 1.02595675,
    "venus": 243.0187,
    "mercury": 58.6462,
    "sol": 25.379995,
    "jupiter": 0.41007,
    "saturn": 0.426,
    "uranus": 0.71833,
    "neptune": 0.67125,
    "pluto": 6.38718,
}

constants = {
    "G" : 6.67408*10**(-11), # Gravitational constant [m^3 kg^-1 s^-2]
    "wE" : 7.2921159*10**(-5), # Earth angular velocity [rad/s]
    "J2E" : 10826269*10**(-3), # Earth J2 constant
}

def proper_round(num, dec=0):  # Add exception check for no decimal point found

    num = str(num)[:str(num).index('.')+dec+2]
    if num[-1]>='5':
        return float(num[:-2-(not dec)]+str(int(num[-2-(not dec)])+1))
    return float(num[:-1])

def polar2cart(r, phi, theta):
    return [
         r * math.sin(phi) * math.cos(theta),
         r * math.sin(theta) * math.sin(phi),
         r * math.cos(phi)
    ]

def rotate(vec, ang, ax='x'):
    if ax == 'x':
        r_x = np.array([[1, 0, 0],
                        [0, math.cos(ang), -1 * math.sin(ang)],
                        [0, math.sin(ang), math.cos(ang)]])
        return np.matmul(r_x, vec)
    elif ax == 'y':
        r_y = np.array([[math.cos(ang), 0, math.sin(ang)],
                        [0, 1, 0],
                        [-math.sin(ang), 0, math.cos(ang)]])
        return np.matmul(r_y, vec)
    elif ax == 'z':
        r_z = np.array([[math.cos(ang), -math.sin(ang), 0],
                        [math.sin(ang), math.cos(ang), 0],
                        [0, 0, 1]])
        return np.matmul(r_z, vec)



import matplotlib.pyplot as plt
from satellite_constellation.Constellation import *
from satellite_constellation.utils import *

def draw_walker(walker_constellation):
    r = walker_constellation.altitude + heavenly_body_radius[walker_constellation.focus]

    t = np.linspace(0, 2 * math.pi, 100)

    x1, y1, z1 = r * np.cos(t), r * np.sin(t), 0 * t

    fig = plt.figure()
    fig.canvas.manager.set_window_title('Walker Constellation')

    perspectives = [[0, 0], [90, 0], [45, 45]]

    ax = [plt.subplot(2, 2, 1, projection='3d'), plt.subplot(2, 2, 2, projection='3d'),
          plt.subplot(2, 2, 3, projection='3d')]
    for idx in range(3):
        ax[idx].view_init(elev=perspectives[idx][0], azim=perspectives[idx][1])
        ax[idx].set_xlim(-r, r)
        ax[idx].set_ylim(-r, r)
        ax[idx].set_zlim(-r, r)
        ax[idx].plot(x1, y1, z1, '--', linewidth=0.1, color='r')  # Plot equatorial circle
        ax[idx].zaxis.set_tick_params(labelsize=3)
        ax[idx].xaxis.set_tick_params(labelsize=3)
        ax[idx].yaxis.set_tick_params(labelsize=3)
        ax[idx].set_xlabel("X", fontsize=3)
        ax[idx].set_ylabel("Y", fontsize=3)
        ax[idx].set_zlabel("Z", fontsize=3)

        for idy in range(walker_constellation.num_planes):  # Plot orbital planes
            ang = idy * 360 / walker_constellation.num_planes
            t = np.linspace(0, 2 * math.pi, 100)
            plt.plot(r * np.cos(t), r * np.sin(t), 0)
            x, y, z = r * np.cos(t), r * np.sin(t), 0 * t
            for idz in range(100):
                coords = np.array([x[idz], y[idz], z[idz]])
                rot_coords = rotate(coords, walker_constellation.inclination * math.pi / 180, 'x')
                rot_coords = rotate(rot_coords, ang * math.pi / 180, 'z')
                x[idz] = rot_coords[0]
                y[idz] = rot_coords[1]
                z[idz] = rot_coords[2]
            ax[idx].plot(x, y, z, '--', linewidth=0.5)

        for idy in range(walker_constellation.num_planes):  # Plot satellites
            for idz in range(walker_constellation.sats_per_plane):
                ctr = idz + idy * (walker_constellation.sats_per_plane)

                x_i, y_i, z_i = sat_x, sat_y, sat_z = polar2cart(r, (90) * math.pi / 180,
                                                                 (walker_constellation.perigee_positions[
                                                                      ctr] +
                                                                  + walker_constellation.ta[ctr]
                                                                  + walker_constellation.raan[ctr]) * math.pi / 180)
                coords = np.array([x_i, y_i, z_i])
                coords = rotate(coords, 90 * math.pi / 180, 'z')
                coords = rotate(coords, walker_constellation.inclination * math.pi / 180, 'x')
                coords = rotate(coords, (walker_constellation.raan[ctr]) * math.pi / 180, 'z')
                ax[idx].scatter(coords[0], coords[1], coords[2])

    return fig

def draw_flower(flower_constellation):
    a = flower_constellation.semi_major
    b = a * math.sqrt(1 - math.pow(flower_constellation.eccentricity, 2))
    f = (flower_constellation.altitude + heavenly_body_radius[flower_constellation.focus]) * 10 ** 3
    disp = a - f

    t = np.linspace(0, 2 * math.pi, 100)

    r = heavenly_body_radius[flower_constellation.focus] * 10 ** 3
    x1, y1, z1 = r * np.cos(t), r * np.sin(t), 0 * t
    x2, y2, z2 = r * np.cos(t), 0 * t, r * np.sin(t)
    x3, y3, z3 = 0*t, r * np.cos(t), r * np.sin(t)

    fig = plt.figure()
    fig.canvas.manager.set_window_title('Flower Constellation')

    r = a

    perspectives = [[0, 0], [90, 0], [45, 45]]

    ax = [plt.subplot(2, 2, 1, projection='3d'), plt.subplot(2, 2, 2, projection='3d'),
          plt.subplot(2, 2, 3, projection='3d')]
    for idx in range(3):
        ax[idx].view_init(elev=perspectives[idx][0], azim=perspectives[idx][1])
        ax[idx].set_xlim(-3 / 2 * a, 3 / 2 * a)
        ax[idx].set_ylim(-3 / 2 * a, 3 / 2 * a)
        ax[idx].set_zlim(-3 / 2 * a, 3 / 2 * a)
        ax[idx].plot(x1, y1, z1, '--', linewidth=0.1, color='r')  # Plot equatorial circle
        ax[idx].plot(x2, y2, z2, '--', linewidth=0.1, color='r')  # Plot equatorial circle
        ax[idx].plot(x3, y3, z3, '--', linewidth=0.1, color='r')  # Plot equatorial circle
        ax[idx].zaxis.set_tick_params(labelsize=3)
        ax[idx].xaxis.set_tick_params(labelsize=3)
        ax[idx].yaxis.set_tick_params(labelsize=3)
        ax[idx].set_xlabel("X", fontsize=3)
        ax[idx].set_ylabel("Y", fontsize=3)
        ax[idx].set_zlabel("Z", fontsize=3)
        for idy in range(min(flower_constellation.num_orbits,flower_constellation.num_satellites)):  # Plot orbital planes
            x, y, z = disp + a * np.cos(t), b * np.sin(t), 0 * t
            for idz in range(100):
                coords = np.array([x[idz], y[idz], z[idz]])
                coords = rotate(coords, flower_constellation.raan[idy] * math.pi / 180, 'z')
                coords = rotate(coords, flower_constellation.inclination * math.pi / 180, 'x')
                x[idz] = coords[0]
                y[idz] = coords[1]
                z[idz] = coords[2]
            ax[idx].plot(x, y, z, '--', linewidth=0.5)

        for idy in range(flower_constellation.num_satellites):  # Plot satellites
            ang = (flower_constellation.true_anomaly[idy] + 180) * math.pi / 180
            x_i, y_i, z_i = disp + a * np.cos(ang), b * np.sin(ang), 0
            coords = np.array([x_i, y_i, z_i])
            coords = rotate(coords, flower_constellation.raan[idy] * math.pi / 180, 'z')
            coords = rotate(coords, flower_constellation.inclination * math.pi / 180, 'x')
            ax[idx].scatter(coords[0], coords[1], coords[2], s=2)

    return fig



"""
Class for holding Constellations of Satellites within it.
"""
from .Satellite import Satellite
from .utils import *
import warnings
import math


class Constellation:
    """
    Class to implement a layer of abstraction for satellite constellations
    Contains parameters general to all types of satellites

    :param num_satellites: Number of satellites used in the constellation
    :param orbital_period: Orbital period of the satellites [s]
    :param altitude: altitude of the satellites [km]
    :param beam_width: Sensor beam width of the satellites [degrees]
    :param eccentricity: Eccentricity of satellite orbits
    :param inclination: Inclination of orbit relative to equatorial plane "i" [degrees]
    :param focus: Heavenly body at the focus of the orbit, defaults to 'Earth'
    """

    def __init__(self, num_satellites, orbital_period, altitude, beam_width, eccentricity, inclination, focus='Earth',
                 name="constellation"):
        self.num_satellites = num_satellites
        self.orbital_period = orbital_period
        self.altitude = altitude
        self.beam_width = beam_width
        self.eccentricity = eccentricity
        self.inclination = inclination
        self.focus = focus
        self.name = name
        self.satellites = []

    def __str__(self):
        sat_string = ""
        for sat in self.satellites:
            sat_string += sat.__str__() + '\n'

        return sat_string.rstrip()

    def as_dict(self):
        constellation = {}
        for sat in self.satellites:
            if sat.name not in constellation:
                constellation[sat.name] = sat.as_dict()
        constellation['Type'] = 'constellation'
        return constellation

    def as_xml(self):
        warnings.warn("XML support is depreciated and not supported from PIGI 0.8.5 onward", DeprecationWarning)
        return self.as_pigi_output()

    def as_pigi_output(self):
        short_scene = ""
        for sat in self.satellites:
            short_scene += sat.as_xml()

        return short_scene


class WalkerConstellation(Constellation):  # Walker delta pattern
    """
    Class for describing and holding a walker constellation of satellites

    :param num_sats: Number of satellites used in the constellation "t"
    :param num_planes: The number of different orbit planes in the constellation "p"
    :param phasing: Dictates the spacing between equivalent satellites in neighbouring orbital planes "f"
    :param inclination: Inclination of orbit relative to equatorial plane "i" [degrees]
    :param altitude: Altitude of satellites in orbit [km]
    :param eccentricity: Eccentricity of satellite orbits
    :param beam_width: Sensor beam width of the satellites [degrees]
    :param name: Name of constellation, defaults to "Sat"
    :param focus: Heavenly body at the focus of the orbit, defaults to 'Earth'
    """

    def __init__(self, num_sats, num_planes, phasing, inclination, altitude,
                 eccentricity, beam_width, name="Sat", focus="earth", starting_number=0):
        super(WalkerConstellation, self).__init__(num_sats, 0, altitude, beam_width, eccentricity, inclination, focus,
                                                  name)
        self.num_planes = num_planes
        self.phasing = phasing
        self.start_num = starting_number
        self.constellation_name = name
        self.sats_per_plane, self.correct_phasing = self.__corrected_planes()
        self.perigee_positions = self.__perigee_positions()
        self.raan = self.__calculate_raan()
        self.ta = self.__calculate_ta()
        self.satellites = self.__build_satellites()

    def __corrected_planes(self):
        sats_per_plane = int(self.num_satellites / self.num_planes)
        corrected_phasing = 360 * self.phasing / self.num_satellites
        return sats_per_plane, corrected_phasing

    def __perigee_positions(self):
        perigees = list(range(0, 360, int(360 / self.sats_per_plane)))
        all_perigees = []
        for i in range(self.num_planes):
            all_perigees.extend(perigees)
        return all_perigees

    def __calculate_raan(self):
        raan = [0] * self.num_satellites
        for i in range(self.sats_per_plane, self.num_satellites):
            raan[i] = raan[i - self.sats_per_plane] + 360 / self.num_planes
        return raan

    def __calculate_ta(self):
        ta = [0] * self.num_satellites
        for i in range(self.sats_per_plane, self.num_satellites):
            ta[i] = ta[i - self.sats_per_plane] + self.correct_phasing
        return ta

    def __build_satellites(self):
        satellites = []
        for i in range(self.num_satellites):
            sat_num = i + self.start_num + 1
            sat_name = self.constellation_name + " " + str(sat_num)
            satellites.append(Satellite(sat_name, self.altitude, self.eccentricity, self.inclination, self.raan[i],
                                        self.perigee_positions[i], self.ta[i], self.beam_width, focus=self.focus,
                                        rads=False))
        return satellites

    # def __calculate_revisit_time(self):  # Method from https://arxiv.org/pdf/1807.02021.pdf
    # longitudinal_
    # return revisit_time

    def __repr__(self):
        return "{0}, {1}, {2}, {3}, {4}, {5}, {6}, name={7}, starting_number={8}".format(self.num_satellites,
                                                                                         self.num_planes,
                                                                                         self.phasing, self.inclination,
                                                                                         self.altitude,
                                                                                         self.eccentricity,
                                                                                         self.beam_width,
                                                                                         self.constellation_name,
                                                                                         self.start_num)

    def representation(self):
        return "{0}:{1}/{2}/{3} Walker".format(self.inclination, self.num_satellites, self.num_planes, self.phasing)


class SOCConstellation(Constellation):  # This is really just a part of a walker star pattern

    """
    Class for describing and holding a walker constellation of satellites

    :param num_streets: Number of satellites used in the constellation
    :param street_width: Angular width of street [degrees]
    :param altitude: Altitude of satellites in orbit [km]
    :param beam_width: Sensor beam width of the satellites [degrees]
    :param raan: List of right ascension for each street [degrees]
    :param eccentricity: Eccentricity of satellite orbits
    :param revisit_time: Time until latitude re-enters satellite coverage [s]
    :param name: Name of constellation, defaults to "Sat"
    :param focus: Heavenly body at the focus of the orbit, defaults to 'Earth'
    """

    def __init__(self, num_streets, street_width, altitude, beam_width, raan, eccentricity, revisit_time, name="Sat",
                 focus="earth", starting_number=0):

        super(SOCConstellation, self).__init__(0, 0, altitude, beam_width, eccentricity, 90, focus, name)

        self.num_streets = num_streets
        self.start_num = starting_number
        self.raan = raan
        self.street_width = street_width
        self.revisit_time = revisit_time
        self.earth_coverage_radius, self.earth_coverage_angle = self.__calculate_earth_coverage()
        self.linear_spacing, self.angular_spacing = self.__calculate_spacing()
        self.perigee, self.semi_major, self.orbital_period = self.__calculate_orbit_params()
        self.num_satellites, self.sats_per_street, self.true_spacing = self.__calculate_required_satellites()
        self.perigee_positions = self.__perigee_positions()
        self.ta = self.__calculate_ta()
        self.satellites = self.__build_satellites()
        self.longitudinal_drift = self.__calculate_longitudinal_drift()

    def __calculate_earth_coverage(self):
        x = self.altitude * math.tan((math.pi / 180) * self.beam_width / 2)
        theta = math.asin(x / heavenly_body_radius[self.focus])
        r = heavenly_body_radius[self.focus] * theta
        return r, theta

    def __calculate_spacing(self):
        street_width = heavenly_body_radius[self.focus] * self.street_width * math.pi / 180
        y = math.sqrt(math.pow(self.earth_coverage_radius, 2) - math.pow(street_width / 2, 2))
        ang_spacing = 2 * y / heavenly_body_radius[self.focus]
        return y, ang_spacing

    def __calculate_orbit_params(self):
        perigee = heavenly_body_radius[self.focus] + self.altitude  # [km]
        semi_major = perigee / (1 - self.eccentricity)  # [km]
        orbital_period = 2 * math.pi * math.sqrt(
            (semi_major * 10 ** 3) ** 3 / (heavenly_body_mass[self.focus] * constants['G']))  # [s]
        return perigee, semi_major, orbital_period

    def __calculate_required_satellites(self):
        num_satellites_a = self.orbital_period / self.revisit_time  # Calculated from revisit time
        num_satellites_b = 2 * math.pi / self.angular_spacing  # Total coverage

        if num_satellites_a > num_satellites_b:
            num_satellites = num_satellites_b
        else:
            num_satellites = num_satellites_a

        upper = math.ceil(num_satellites)
        lower = math.floor(num_satellites)

        if num_satellites - lower <= 0.01:
            true_spacing = 360 / lower
            total_satellites = lower * self.num_streets
            return total_satellites, lower, true_spacing
        else:
            true_spacing = 360 / upper
            total_satellites = upper * self.num_streets
            return total_satellites, upper, true_spacing

    def __calculate_ta(self):
        phase = 360 / self.num_satellites
        ta = [0] * self.num_satellites
        for i in range(self.num_streets):
            for j in range(self.sats_per_street):
                ta[i * self.sats_per_street + j] = i * phase
        return ta

    def __perigee_positions(self):
        perigees = list(range(0, 360, int(360 / self.sats_per_street)))
        all_perigees = []
        for i in range(self.num_satellites):
            all_perigees.extend(perigees)
        return all_perigees

    def __build_satellites(self):
        satellites = []
        for i in range(self.num_streets):
            for j in range(self.sats_per_street):
                sat_num = i * self.sats_per_street + j + self.start_num + 1
                sat_name = self.name + " " + str(sat_num)
                satellites.append(Satellite(sat_name, self.altitude, self.eccentricity, self.inclination, self.raan[i],
                                            self.perigee_positions[j], self.ta[i * self.sats_per_street + j],
                                            self.beam_width, focus=self.focus, rads=False))
        return satellites

    def __calculate_longitudinal_drift(self):
        drift = self.orbital_period * constants["wE"] * 180 / math.pi
        return drift

    def __repr__(self):
        return "{0}, {1}, {2}, {3}, {4}".format(self.num_satellites,
                                                self.altitude, self.eccentricity,
                                                self.beam_width,
                                                self.name)

    def representation(self):
        return "{0}:{1}/{2}/{3} Walker".format(self.inclination, self.num_satellites, self.num_streets, 0)


class FlowerConstellation(Constellation):
    """
    Class for describing and holding a flower constellation of satellites

    :param num_petals: Number of petals formed when viewing the relative orbits
    :param num_days: The number of days for the constellation to completely repeat its orbit
    :param num_satellites: The desired number of satellites involved in the constellation
    :param phasing_n: Phasing parameter n, effects allowable satellite positions
    :param phasing_d: Phasing parameter d, effects the maximum number of satellites
    :param perigee_argument: Argument of perigee for satellite orbits [degrees]
    :param inclination: Inclination of orbit relative to equatorial plane [degrees]
    :param perigee_altitude: Altitude of perigee for satellite orbits [km]
    :param beam_width: Angular width of satellite sensor beam [degrees]
    :param focus: Object at focus of orbit, defaults to 'earth'
    """

    def __init__(self, num_petals, num_days, num_satellites, phasing_n, phasing_d, perigee_argument,
                 inclination, perigee_altitude, beam_width, focus='earth', name="constellation"):
        super(FlowerConstellation, self).__init__(num_satellites, 0, perigee_altitude, beam_width, 0, inclination,
                                                  focus, name)
        self.num_petals = num_petals
        self.num_days = num_days
        self.phasing_n = phasing_n
        self.phasing_d = phasing_d
        self.perigee_argument = perigee_argument
        self.max_sats_per_orbit, self.max_sats = self.__calculate_max_satellites()
        self.raan_spacing, self.mean_anomaly_spacing = self.__calculate_spacing()
        self.num_orbits = self.__calculate_num_orbits()
        self.orbital_period, self.eccentricity, self.semi_major = self.__calculate_orbit_params()
        self.raan, self.mean_anomaly, self.true_anomaly = self.__calculate_orbits()
        self.revisit_time = self.__calculate_revisit_time()
        self.minimum_revisit_time = self.__calculate_minimum_revisit_time()
        self.satellites = self.__build_satellites()

    def __calculate_max_satellites(self):
        max_sats = self.phasing_d * self.num_days
        if max_sats < self.num_satellites:
            self.num_satellites = max_sats
        return self.num_days, max_sats

    def __calculate_num_orbits(self):
        # return math.floor(self.num_satellites / self.num_days)
        return self.phasing_d

    def __calculate_spacing(self):
        raan_spacing = -360 * self.phasing_n / self.phasing_d
        mean_anomaly_spacing = -1 * raan_spacing * self.num_petals / self.num_days
        if abs(mean_anomaly_spacing) > 360:
            mean_anomaly_spacing = mean_anomaly_spacing % 360
        if abs(raan_spacing) > 360:
            raan_spacing %= 360

        return raan_spacing, mean_anomaly_spacing

    def __calculate_orbits(self):
        raan = [0]
        M = [0]
        v = [0]
        testList = [[0, 0]]
        for idx in range(1, min(self.max_sats, self.num_satellites)):
            raan_i = (raan[idx - 1] + self.raan_spacing)
            if raan_i < 0:
                raan_i = 360 + raan_i
            raan.append(raan_i)
            M_i = M[idx - 1] + self.mean_anomaly_spacing
            if (abs(M_i) > 360):
                M_i = M_i % 360
            if M_i < 0:
                M_i = 360 + M_i
            M.append(M_i)
            v_i = M_i + (2 * self.eccentricity - 0.25 * math.pow(self.eccentricity, 3)) * math.sin(M_i * math.pi / 180)
            v.append(v_i)
        return raan, M, v

    def __calculate_orbit_params(self):
        M = heavenly_body_mass[self.focus]
        G = constants["G"]
        wE = constants["wE"]
        T = (2 * math.pi / wE) * (self.num_days / self.num_petals)
        a = math.pow(G * M * math.pow(T / (2 * math.pi), 2), 1 / 3)
        e = 1 - (heavenly_body_radius[self.focus] * 10 ** 3 + self.altitude) / a

        return T, e, a

    def __calculate_revisit_time(self):
        revisit_time = self.num_days / self.num_satellites
        return revisit_time

    def __calculate_minimum_revisit_time(self):
        min_revisit_time = self.num_days / self.max_sats
        return min_revisit_time

    def __build_satellites(self):
        satellites = []
        for i in range(self.num_satellites):
            sat_name = i
            satellites.append(Satellite(sat_name, self.altitude, self.eccentricity, self.inclination, self.raan[i],
                                        self.true_anomaly[i], self.true_anomaly[i], self.beam_width, self.focus,
                                        rads=False))

        return satellites

    def representation(self, representation_type="flower"):
        if representation_type == "flower":
            i = self.inclination
            return "{0}-{1}-{2}-{3}-{4} Flower".format(self.num_petals, self.num_days, self.num_satellites,
                                                       self.phasing_n,
                                                       self.phasing_d)
        elif representation_type == "walker":
            i = self.inclination
            No = self.phasing_d
            G = self.phasing_d * self.num_days / self.max_sats
            Nso = self.max_sats_per_orbit
            t = No * Nso
            p = No
            Nc = (self.num_petals * self.phasing_n + self.phasing_d) / G
            f = int(Nc % No)

            return "{0}:{1}/{2}/{3} Walker".format(i, t, p, f)



"""
A class for creating a satellite object, describing the characteristics of it.
"""

from math import pi
from .utils import heavenly_body_radius
import warnings


class Satellite(object):

    def __init__(self, name, altitude, eccentricity, inclination, right_ascension, perigee, ta, beam,
                 focus="earth", rads=True):
        self._name = name
        self._altitude = altitude
        self._focus = focus
        self._true_alt = self.altitude + self.__get_radius()
        self._eccentricity = eccentricity
        self._beam = beam

        if not rads:
            self.inclination = inclination
            self.right_ascension = right_ascension
            self.perigee = perigee
            self.ta = ta
            self.inclination_r, self.right_ascension_r, self.perigee_r, self.ta_r = self.__convert_to_rads()
        else:
            self.inclination_r = inclination
            self.right_ascension_r = right_ascension
            self.perigee_r = perigee
            self.ta_r = ta
            self.inclination, self.right_ascension, self.perigee, self.ta = self.__convert_to_degs()

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        self._name = new_name

    @property
    def altitude(self):
        return self._altitude

    @altitude.setter
    def altitude(self, new_alt):
        if new_alt < 100:
            return ValueError("Satellite's orbital altitude must be over the Karman line.")
        else:
            self._altitude = new_alt
            self._true_alt = new_alt + self.__get_radius()

    @property
    def true_alt(self):
        return self._true_alt

    @property
    def eccentricity(self):
        return self._eccentricity

    @eccentricity.setter
    def eccentricity(self, new_e):
        if new_e < 0:
            return ValueError("Eccentricity can't be set below a perfect circle.")
        else:
            self._eccentricity = new_e

    @property
    def beam(self):
        return self._beam

    @beam.setter
    def beam(self, new_beam):
        if new_beam < 0:
            return ValueError("Beam width must be between 0 and 180 degrees")
        elif new_beam > 180:
            return ValueError("Beam width must be between 0 and 180 degrees")
        else:
            self._beam = new_beam

    def __convert_to_rads(self, value=None):
        to_rad = pi / 180
        if value:
            return value * to_rad
        else:
            return self.inclination * to_rad, self.right_ascension * to_rad, self.perigee * to_rad, self.ta * to_rad

    def __convert_to_degs(self, value=None):
        to_deg = 180 / pi
        if value:
            return value * to_deg
        else:
            return self.inclination_r * to_deg, self.right_ascension_r * to_deg, self.perigee_r * to_deg, \
                   self.ta_r * to_deg

    def __get_radius(self):
        return heavenly_body_radius[self._focus.lower()]

    def __repr__(self):
        return "{0}, {1}, {2}, {3}, {4}, {5}, {6}".format(self.name, self.altitude, self.eccentricity,
                                                          self.inclination, self.right_ascension, self.perigee, self.ta)

    def __str__(self):
        return "Satellite Name: {0}, Alt: {1}, e: {2}, " \
               "Inclination: {3}, RA: {4}, Periapsis: {5}, Anomaly: {6}".format(self.name, self.altitude,
                                                                                self.eccentricity, self.inclination,
                                                                                self.right_ascension, self.perigee,
                                                                                self.ta)

    def as_dict(self, rads=True):
        if rads:
            sat = {"Name": self.name,
                   "Orbital Elements": {
                       "Eccentricity": self.eccentricity,
                       "Right Ascension": self.right_ascension_r,
                       "Semi-major Axis": self.true_alt,
                       "Arg. Periapsis": self.perigee_r,
                       "Mean Anomaly": self.ta_r,
                       "Inclination": self.inclination_r
                   },
                   "Beam Width": self.beam}
        else:
            sat = {"Name": self.name,
                   "Orbital Elements": {
                       "Eccentricity": self.eccentricity,
                       "Right Ascension": self.right_ascension,
                       "Semi-major Axis": self.true_alt,
                       "Arg. Periapsis": self.perigee,
                       "Mean Anomaly": self.ta,
                       "Inclination": self.inclination
                   },
                   "Beam Width": self.beam}
        sat['Focus'] = self._focus
        sat['Type'] = 'satellite'

        return sat

    def as_xml(self, epoch_date='2017-Jan-18 00:00:00', fov=1):
        warnings.warn("XML support is depreciated and not supported from PIGI 0.8.5 onward", DeprecationWarning)
        return '\t\t< Entity Type = "Satellite" Name = "{0}" >\n' \
               '\t\t\t<PropertySection Name="UserProperties">\n' \
               '\t\t\t\t<StringPropertyValue name="PlanetName" value="Earth"/>\n' \
               '\t\t\t\t<StringPropertyValue name="CatalogName" value="{0}"/>\n' \
               '\t\t\t\t<StringPropertyValue name="MeshName" value="SaberBox.mesh"/>\n' \
               '\t\t\t\t<StringPropertyValue name="BindingsFile" value=""/>\n' \
               '\t\t\t\t<IntPropertyValue name="ManualOrbitalElements" value="0"/>\n' \
               '\t\t\t\t<StringPropertyValue name="AssemblyFile" value=""/>\n' \
               '\t\t\t\t<StringPropertyValue name="SystemMapSourceId" value=""/>\n' \
               '\t\t\t\t<StringPropertyValue name="ResourceGroup" value="Autodetect"/>\n' \
               '\t\t\t\t<StringPropertyValue name="MetricSourceIds" value=""/>\n' \
               '\t\t\t\t<FloatPropertyValue name="BeamWidth" value="{1}"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="SGP4 Parameters">\n' \
               '\t\t\t\t<FloatPropertyValue name="B Star" value="-1.1606e-005"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="Eccentricity" value="{2}"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="RAAN" value="{3}"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="Semi-major axis" value="{4}"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="Arg. Perigee" value="{5}"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="Mean Anomaly" value="{6}"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="Inclination" value="{7}"/>\n' \
               '\t\t\t\t<TimestampPropertyValue name="Epoch" value="{8}"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="Fov">\n' \
               '\t\t\t\t<EnumPropertyValue name="Enabled" value="{9}"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="TargetSceneNode">\n' \
               '\t\t\t\t<ArrayPropertyValue name="Position" value="[0, 0, 0]"/>\n' \
               '\t\t\t\t<ArrayPropertyValue name="Orientation" value="[1, 0, 0, 0]"/>\n' \
               '\t\t\t\t<ArrayPropertyValue name="Scale" value="[200, 200, 200]"/>\n' \
               '\t\t\t\t<EnumPropertyValue name="Debug" value="0"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="Billboard">\n' \
               '\t\t\t\t<ArrayPropertyValue name="Position" value="[0, 0, 0]"/>\n' \
               '\t\t\t\t<ArrayPropertyValue name="Orientation" value="[1, 0, 0, 0]"/>\n' \
               '\t\t\t\t<ArrayPropertyValue name="Scale" value="[200, 200, 200]"/>\n' \
               '\t\t\t\t<EnumPropertyValue name="Debug" value="0"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="Favourite">\n' \
               '\t\t\t\t<EnumPropertyValue name="favourite" value="0"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t</Entity>\n'.format(self.name, self.beam, self.eccentricity, self.right_ascension_r,
                                        self.true_alt, self.perigee_r, self.ta_r, self.inclination_r,
                                        epoch_date, fov)



"""
Recreation of the Octave SceneCreator script for use within Flask or other python projects
@creator: Aidan O'Brien
"""

from .Constellation import WalkerConstellation, SOCConstellation
from .GroundStation import GroundStation
from itertools import zip_longest
import warnings
from .utils import mod, heavenly_body_radius
from .ConstellationExceptions import *



def scene_xml_generator(scene):
    warnings.warn("XML support is depreciated and not supported from PIGI 0.8.5 onward", DeprecationWarning)
    scene_start = '<Pigi>\n' \
                  '\t<Entities>\n' \
                  '\t\t<Entity Type="Planet" Name="Mercury">\n' \
                  '\t\t\t<PropertySection Name="UserProperties">\n' \
                  '\t\t\t\t<StringPropertyValue name="PlanetName" value="Mercury"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="obliquity" value="0.1"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="scale" value="0.244"/>\n' \
                  '\t\t\t\t<IntPropertyValue name="Atmosphere" value="0"/>\n' \
                  '\t\t\t</PropertySection>\n' \
                  '\t\t</Entity>\n' \
                  '\t\t<Entity Type="Planet" Name="Venus">\n' \
                  '\t\t\t<PropertySection Name="UserProperties">\n' \
                  '\t\t\t\t<StringPropertyValue name="PlanetName" value="Venus"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="obliquity" value="177"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="scale" value="0.605"/>\n' \
                  '\t\t\t\t<IntPropertyValue name="Atmosphere" value="1"/>\n' \
                  '\t\t\t</PropertySection>\n' \
                  '\t\t</Entity>\n' \
                  '\t\t<Entity Type="Planet" Name="Earth">\n' \
                  '\t\t\t<PropertySection Name="UserProperties">\n' \
                  '\t\t\t\t<StringPropertyValue name="PlanetName" value="Earth"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="obliquity" value="23.4"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="scale" value="0.6371"/>\n' \
                  '\t\t\t\t<IntPropertyValue name="Atmosphere" value="1"/>\n' \
                  '\t\t\t</PropertySection>\n' \
                  '\t\t</Entity>\n' \
                  '\t\t<Entity Type="Planet" Name="Mars">\n' \
                  '\t\t\t<PropertySection Name="UserProperties">\n' \
                  '\t\t\t\t<StringPropertyValue name="PlanetName" value="Mars"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="obliquity" value="25"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="scale" value="0.3371"/>\n' \
                  '\t\t\t\t<IntPropertyValue name="Atmosphere" value="1"/>\n' \
                  '\t\t\t</PropertySection>\n' \
                  '\t\t</Entity>\n' \
                  '\t\t<Entity Type="Planet" Name="Jupiter">\n' \
                  '\t\t\t<PropertySection Name="UserProperties">\n' \
                  '\t\t\t\t<StringPropertyValue name="PlanetName" value="Jupiter"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="obliquity" value="3"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="scale" value="6.99"/>\n' \
                  '\t\t\t\t<IntPropertyValue name="Atmosphere" value="0"/>\n' \
                  '\t\t\t</PropertySection>\n' \
                  '\t\t</Entity>\n' \
                  '\t\t<Entity Type="Planet" Name="Saturn">\n' \
                  '\t\t\t<PropertySection Name="UserProperties">\n' \
                  '\t\t\t\t<StringPropertyValue name="PlanetName" value="Saturn"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="obliquity" value="27"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="scale" value="6.033"/>\n' \
                  '\t\t\t\t<IntPropertyValue name="Atmosphere" value="0"/>\n' \
                  '\t\t\t</PropertySection>\n' \
                  '\t\t</Entity>\n' \
                  '\t\t<Entity Type="Planet" Name="Neptune">\n' \
                  '\t\t\t<PropertySection Name="UserProperties">\n' \
                  '\t\t\t\t<StringPropertyValue name="PlanetName" value="Neptune"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="obliquity" value="30"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="scale" value="2.46"/>\n' \
                  '\t\t\t\t<IntPropertyValue name="Atmosphere" value="0"/>\n' \
                  '\t\t\t</PropertySection>\n' \
                  '\t\t</Entity>\n' \
                  '\t\t<Entity Type="Planet" Name="Pluto">\n' \
                  '\t\t\t<PropertySection Name="UserProperties">\n' \
                  '\t\t\t\t<StringPropertyValue name="PlanetName" value="Pluto"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="obliquity" value="120"/>\n' \
                  '\t\t\t\t<FloatPropertyValue name="scale" value="0.186"/>\n' \
                  '\t\t\t\t<IntPropertyValue name="Atmosphere" value="0"/>\n' \
                  '\t\t\t</PropertySection>\n' \
                  '\t\t</Entity>\n'

    scene_end = '\t</Entities>\n' \
                '</Pigi>'

    scene_xml = ""
    for item in scene:
        scene_xml += item.as_xml()

    return "{0} {1} {2}".format(scene_start, scene_xml, scene_end)


def constellation_creator(num_constellations, satellite_nums, satellite_planes, plane_phasing, inclination, altitude,
                          eccentricity, constellation_beam_width, sat_name="Sat", focus="earth"):
    """

    Need to add the streets of coverage method to the scene creator
    Change this to use 3 param
    Number of constellations
    List of constellation types
    List of dict with constellation vars, dict change with different type

    :param num_constellations: Integer of the number of constellations that are for the scene
    :param satellite_nums: List of numbers of satellites for each constellation
    :param satellite_planes: List of number of planes of satellites for each constellation.
    :param plane_phasing: The planar phases for their respective constellations
    :param inclination: List of inclinations for constellations
    :param altitude: List of altitudes for constellations
    :param eccentricity: List of eccentricities for each constellation
    :param constellation_beam_width: List of Beam widths for satellites in each constellation
    :param sat_name: Root name for satellites, defaults to Sat
    :param focus: The focus of the satellite, i.e. Earth, Luna, Mars, in lower case.
    :return: Returns a list of constellations that have been formatted
    """

    walker_errors(num_constellations, satellite_nums, satellite_planes, plane_phasing, inclination, altitude,
                  eccentricity, constellation_beam_width, focus="earth")

    scene = []
    for idx in range(num_constellations):
        start_num = sum(satellite_nums[0:idx])
        scene.append(
            WalkerConstellation(satellite_nums[idx], satellite_planes[idx], plane_phasing[idx], inclination[idx],
                                altitude[idx], eccentricity[idx], constellation_beam_width[idx], name=sat_name,
                                starting_number=start_num, focus=focus))

    return scene


def ground_array_creator(num_ground_stations, latitudes, longitudes, elevations, beam_widths, name="GS"):
    """

    :param num_ground_stations:
    :param latitudes:
    :param longitudes:
    :param elevations:
    :param beam_widths:
    :param name:
    :return:
    """

    if num_ground_stations < 1 or (num_ground_stations % 1):
        return -1

    if num_ground_stations != len(latitudes):
        return -1

    try:
        zip_longest(latitudes, longitudes, elevations)
    except IndexError:
        return -1

    if (len(latitudes) != len(beam_widths)) or (len(beam_widths) == 1):
        return -1

    scene = []
    for idx in range(num_ground_stations):
        gs_name = name + str(idx)
        if len(beam_widths) > 1:
            beam = beam_widths[idx]
        else:
            beam = beam_widths
        scene.append(GroundStation(gs_name, latitudes[idx], longitudes[idx], elevations[idx], beam))

    return scene


def walker_errors(num_constellations, satellite_nums, satellite_planes, plane_phasing, inclination, altitude,
                  eccentricity,
                  constellation_beam_width, sat_name="Sat", focus="earth"):
    error_found = False

    if num_constellations < 1 or (num_constellations % 1):
        error_found = True
        raise ConstellationNumberError("Invalid integer number of constellations")

    if num_constellations != len(satellite_nums):
        error_found = True
        raise ConstellationConfigurationError("Number of constellations does not match number of satellites provided")

    if any(mod(satellite_nums, satellite_planes)):
        error_found = True
        raise ConstellationPlaneMismatchError("Number of satellites not compatible with planes in constellation")

    for idx, phase in enumerate(plane_phasing):
        if phase < 0:
            error_found = True
            raise PhaseError("Negative number of phases not allowed")
        # elif phase >= satellite_planes[idx]:
        #     raise PhaseError("Number of phases in constellation larger than number of planes")
        # elif (satellite_nums[idx] % phase) != 0:
        #     raise PhaseError("Number of satellites must be divisible by the number of phases")
        elif phase % 1:
            error_found = True
            raise PhaseError("Number of phases not an integer")
        elif phase % satellite_planes[idx] == 0:
            error_found = True
            raise PhaseError("Phase cannot equal number of planes times a constant")

    if any(abs(x) > 90 for x in inclination):
        error_found = True
        raise InclinationError("Inclination greater than 90 degrees from equitorial orbit")
        # Could implement wrap around of angles here

    if any(x < 0 for x in altitude):
        error_found = True
        raise AltitudeError("Negative altitude not allowed")
    elif any(x < 100 for x in altitude):
        error_found = True
        raise AltitudeError("Altitude below Karman line")

    if any(x < 0 for x in eccentricity):
        error_found = True
        raise EccentricityError("Invalid eccentricity. Eccentricity cannot be less than 0")
    elif any(x >= 1 for x in eccentricity):
        error_found = True
        raise EccentricityError("Invalid eccentricity. Hyperbolic trajectories not allowed")

    if any(x <= 0 or x > 180 for x in constellation_beam_width):
        error_found = True
        raise BeamError("Beam width error. Must be within 0 to 180 degrees")

    if focus.lower() not in heavenly_body_radius:
        error_found = True
        raise FocusError("'" + focus.capitalize() + "' not supported as a celestial body origin.")

    if error_found:
        return True
    else:
        return False


def create_scene(num_constellations, num_sats, sat_planes, plane_phasing, sat_inclination, sat_alt,
                 sat_eccentricity, const_beam_width,
                 num_ground_stations, latitudes, longitudes, elevations, beam_widths,
                 sat_name="Sat", gsname="GS"):
    """
    Creates the scene list from constellations and ground stations
    :param num_constellations:
    :param num_sats:
    :param sat_planes:
    :param plane_phasing:
    :param sat_inclination:
    :param sat_altitude:
    :param sat_eccentricity:
    :param const_beam_width:
    :param num_ground_stations:
    :param latitudes:
    :param longitudes:
    :param elevations:
    :param beam_widths: List of beam widths for the ground stations
    :param sat_name: Root of satellite names
    :param gsname: Root of ground station names
    :return: List of constellation and ground station objects
    """

    scene = constellation_creator(num_constellations, num_sats, sat_planes, plane_phasing, sat_inclination, sat_alt,
                                  sat_eccentricity, const_beam_width, sat_name=sat_name)
    scene.extend(ground_array_creator(num_ground_stations, latitudes, longitudes, elevations, beam_widths, name=gsname))

    return scene






from satellite_constellation.visualiser import *
import matplotlib.pyplot as plt
import argparse

def argParser():
    """
    Args:
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--test', dest='test',
                        action="store_true",default=False,
                        help="Run test constellations")
    return parser.parse_args()

def main():
    p = argParser()
    if p.test:
        draw_walker(WalkerConstellation(8, 2, 1, 45, 1000, 0, 20))
        draw_flower(FlowerConstellation(37, 18, 57, 6, 19, 0, 0, 19702, 30))
        plt.show()
    else:
        print('Nothing to do') #TODO add more functionality here

if __name__ == '__main__':
    main()



import warnings


class GroundStation(object):
    def __init__(self, name, lat, long, elevation, beam_width):
        self.__name = name
        self.__lat = lat
        self.__long = long
        self.__elevation = elevation
        self.__beam = beam_width

    @property
    def name(self):
        return self.__name

    @name.setter
    def name(self, new_name):
        self.__name = new_name

    @property
    def lat(self):
        return self.__lat

    @lat.setter
    def lat(self, new_lat):
        if (new_lat < -90) or (new_lat > 90):
            return ValueError("Latitude must be between -90 and 90")
        else:
            self.__lat = new_lat

    @property
    def long(self):
        return self.__long

    @long.setter
    def long(self, new_long):
        if (new_long < -180) or (new_long > 180):
            return ValueError("Longitude must be between -180 and 180")
        else:
            self.__long = new_long

    @property
    def elevation(self):
        return self.__elevation

    @elevation.setter
    def elevation(self, new_elev):
        if new_elev < 0:
            return ValueError("Elevation must be above 0")
        if new_elev > 8900:
            return ValueError("Elevation must be on the ground")
        else:
            self.__elevation = new_elev

    @property
    def beam(self):
        return self.__beam

    @beam.setter
    def beam(self, new_beam):
        if (new_beam < 0) or (new_beam > 180):
            return ValueError("Beam width must be between 0 and 180 degrees")
        self.__beam = new_beam

    def as_xml(self):
        warnings.warn("XML support is depreciated and not supported from PIGI 0.8.5 onward", DeprecationWarning)
        return '\t\t<Entity Type="GroundStation" Name="{0}">\n' \
               '\t\t\t<PropertySection Name="UserProperties">\n' \
               '\t\t\t\t<FloatPropertyValue name="Latitude" value="{1}"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="Longitude" value="{2}"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="Elevation" value="{3}"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="BeamWidth" value="{4}"/>\n' \
               '\t\t\t\t<StringPropertyValue name="PlanetName" value="Earth"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="Animation">\n' \
               '\t\t\t\t<ArrayPropertyValue name="Position" value="[6339.69, -699.193, 0]"/>\n' \
               '\t\t\t\t<ArrayPropertyValue name="Orientation" value="[1, 0, 0, 0]"/>\n' \
               '\t\t\t\t<ArrayPropertyValue name="Scale" value="[1, 1, 1]"/>\n' \
               '\t\t\t\t<EnumPropertyValue name="Debug" value="0"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="Time Input">\n' \
               '\t\t\t\t<TimestampPropertyValue name="Timepoint" value="2016-May-07 08:32:21.059611"/>\n' \
               '\t\t\t\t<DurationPropertyValue name="Duration" value="2"/>\n' \
               '\t\t\t\t<DurationPropertyValue name="StartOffset" value="-1"/>\n' \
               '\t\t\t\t<DurationPropertyValue name="Timestep" value="0.000694444"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="Favourite">\n' \
               '\t\t\t\t<EnumPropertyValue name="favourite" value="0"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="Mesh">\n' \
               '\t\t\t\t<ArrayPropertyValue name="Position" value="[0, 0, 0]"/>\n' \
               '\t\t\t\t<ArrayPropertyValue name="Orientation" value="[1, 0, 0, 0]"/>\n' \
               '\t\t\t\t<ArrayPropertyValue name="Scale" value="[500, 500, 500]"/>\n' \
               '\t\t\t\t<StringPropertyValue name="name" value="GroundStation.mesh"/>\n' \
               '\t\t\t\t<EnumPropertyValue name="Debug" value="0"/>\n' \
               '\t\t\t\t<StringPropertyValue name="group" value="SolarSystem"/>\n' \
               '\t\t\t\t<EnumPropertyValue name="visibility" value="0"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t\t<PropertySection Name="Look Angles">\n' \
               '\t\t\t\t<ArrayPropertyValue name="LatLon" value="[0, 0]"/>\n' \
               '\t\t\t\t<FloatPropertyValue name="Elevation" value="0"/>\n' \
               '\t\t\t</PropertySection>\n' \
               '\t\t</Entity>\n'.format(self.name, self.lat, self.long, self.elevation, self.beam)

    def __repr__(self):
        return "{0}, {1}, {2}, {3}, {4}".format(self.name, self.lat, self.long, self.elevation, self.beam)

    def __str__(self):
        return "Ground Station: {0}\n" \
               "Latitude: {1}, Longitude: {2}" \
               "Elevation: {3}, Beam Width: [4}".format(self.name, self.lat, self.long, self.elevation, self.beam)

    def as_dict(self):
        return {"Name": self.name,
                "Latitude": self.lat,
                "Longitude": self.long,
                "Elevation": self.elevation,
                "Beam Width": self.beam,
                "Type": 'station'}



from setuptools import find_packages, setup

setup(name="satellite-constellation",
    version="0.2",
    description="Create Satellite Constellations around the Earth, Luna and planets in the solar system",
    long_description=open('README.rst').read(),
    author="Aidan O'Brien - Saber Astronautics Australia",
    author_email="aidan.obrien@saberastro.com",
    url="https://github.com/SaberAidan/SatelliteConstellationCreator",
    license='MIT',
    entry_points={
        'console_scripts': ['satellite-constellation=satellite_constellation.main:main'],
    },
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    package_dir={'satellite_constellation': 'satellite_constellation'},
    keywords="satellite-constellation satellite satellites constellation orbital mechanics",
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Scientific/Engineering :: Astronomy',
        'Topic :: Scientific/Engineering :: Physics'
    ])





class ConstellationNumberError(Exception):
    pass


class ConstellationConfigurationError(Exception):
    pass


class ConstellationPlaneMismatchError(Exception):
    pass


class PhaseError(Exception):
    pass


class InclinationError(Exception):
    pass


class AltitudeError(Exception):
    pass


class EccentricityError(Exception):
    pass


class BeamError(Exception):
    pass


class FocusError(Exception):
    pass
